# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/data_source.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from tecton_proto.args import data_source_config_pb2 as tecton__proto_dot_args_dot_data__source__config__pb2
from tecton_proto.args import diff_options_pb2 as tecton__proto_dot_args_dot_diff__options__pb2
from tecton_proto.args import user_defined_function_pb2 as tecton__proto_dot_args_dot_user__defined__function__pb2
from tecton_proto.common import spark_schema_pb2 as tecton__proto_dot_common_dot_spark__schema__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/args/data_source.proto',
  package='tecton_proto.args',
  syntax='proto2',
  serialized_options=b'\n\017com.tecton.argsP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n#tecton_proto/args/data_source.proto\x12\x11tecton_proto.args\x1a\x1egoogle/protobuf/duration.proto\x1a*tecton_proto/args/data_source_config.proto\x1a$tecton_proto/args/diff_options.proto\x1a-tecton_proto/args/user_defined_function.proto\x1a&tecton_proto/common/spark_schema.proto\"{\n\x1b\x44\x61tetimePartitionColumnArgs\x12\x1f\n\x0b\x63olumn_name\x18\x01 \x01(\tR\ncolumnName\x12\x1a\n\x08\x64\x61tepart\x18\x02 \x01(\tR\x08\x64\x61tepart\x12\x1f\n\x0bzero_padded\x18\x03 \x01(\x08R\nzeroPadded\"\xd0\x03\n\x12HiveDataSourceArgs\x12\x14\n\x05table\x18\x01 \x01(\tR\x05table\x12\x1a\n\x08\x64\x61tabase\x18\x02 \x01(\tR\x08\x64\x61tabase\x12\x32\n\x15\x64\x61te_partition_column\x18\x03 \x01(\tR\x13\x64\x61tePartitionColumn\x12\x32\n\x15timestamp_column_name\x18\x04 \x01(\tR\x13timestampColumnName\x12)\n\x10timestamp_format\x18\x05 \x01(\tR\x0ftimestampFormat\x12\'\n\x0fskip_validation\x18\x06 \x01(\x08R\x0eskipValidation\x12l\n\x1a\x64\x61tetime_partition_columns\x18\x07 \x03(\x0b\x32..tecton_proto.args.DatetimePartitionColumnArgsR\x18\x64\x61tetimePartitionColumns\x12X\n\x14raw_batch_translator\x18\t \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x12rawBatchTranslatorJ\x04\x08\x08\x10\t\"\xac\x03\n\x12\x46ileDataSourceArgs\x12\x10\n\x03uri\x18\x01 \x01(\tR\x03uri\x12\x1f\n\x0b\x66ile_format\x18\x02 \x01(\tR\nfileFormat\x12\x33\n\x16\x63onvert_to_glue_format\x18\x03 \x01(\x08R\x13\x63onvertToGlueFormat\x12X\n\x14raw_batch_translator\x18\x04 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x12rawBatchTranslator\x12$\n\nschema_uri\x18\x05 \x01(\tB\x05\x92M\x02\x08\x01R\tschemaUri\x12\x32\n\x15timestamp_column_name\x18\x06 \x01(\tR\x13timestampColumnName\x12)\n\x10timestamp_format\x18\x07 \x01(\tR\x0ftimestampFormat\x12I\n\x0fschema_override\x18\t \x01(\x0b\x32 .tecton_proto.common.SparkSchemaR\x0eschemaOverrideJ\x04\x08\x08\x10\t\"0\n\x06Option\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value\"\x98\x04\n\x15KinesisDataSourceArgs\x12\x1f\n\x0bstream_name\x18\x01 \x01(\tR\nstreamName\x12\x16\n\x06region\x18\x02 \x01(\tR\x06region\x12Z\n\x15raw_stream_translator\x18\x03 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x13rawStreamTranslator\x12#\n\rtimestamp_key\x18\x04 \x01(\tR\x0ctimestampKey\x12o\n\x1f\x64\x65\x66\x61ult_initial_stream_position\x18\x05 \x01(\x0e\x32(.tecton_proto.args.InitialStreamPositionR\x1c\x64\x65\x66\x61ultInitialStreamPosition\x12\x64\n!default_watermark_delay_threshold\x18\x06 \x01(\x0b\x32\x19.google.protobuf.DurationR\x1e\x64\x65\x66\x61ultWatermarkDelayThreshold\x12\x33\n\x15\x64\x65\x64uplication_columns\x18\x07 \x03(\tR\x14\x64\x65\x64uplicationColumns\x12\x33\n\x07options\x18\x08 \x03(\x0b\x32\x19.tecton_proto.args.OptionR\x07optionsJ\x04\x08\t\x10\n\"\xcd\x05\n\x13KafkaDataSourceArgs\x12\x36\n\x17kafka_bootstrap_servers\x18\x01 \x01(\tR\x15kafkaBootstrapServers\x12\x16\n\x06topics\x18\x02 \x01(\tR\x06topics\x12Z\n\x15raw_stream_translator\x18\x03 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x13rawStreamTranslator\x12#\n\rtimestamp_key\x18\x04 \x01(\tR\x0ctimestampKey\x12\x64\n!default_watermark_delay_threshold\x18\x05 \x01(\x0b\x32\x19.google.protobuf.DurationR\x1e\x64\x65\x66\x61ultWatermarkDelayThreshold\x12\x33\n\x07options\x18\x06 \x03(\x0b\x32\x19.tecton_proto.args.OptionR\x07options\x12\x39\n\x15ssl_keystore_location\x18\x07 \x01(\tB\x05\x92M\x02\x08\x01R\x13sslKeystoreLocation\x12K\n\x1fssl_keystore_password_secret_id\x18\x08 \x01(\tB\x05\x92M\x02\x08\x01R\x1bsslKeystorePasswordSecretId\x12=\n\x17ssl_truststore_location\x18\t \x01(\tB\x05\x92M\x02\x08\x01R\x15sslTruststoreLocation\x12O\n!ssl_truststore_password_secret_id\x18\n \x01(\tB\x05\x92M\x02\x08\x01R\x1dsslTruststorePasswordSecretId\x12\x32\n\x11security_protocol\x18\x0b \x01(\tB\x05\x92M\x02\x08\x01R\x10securityProtocol\"\xf3\x01\n\x16RedshiftDataSourceArgs\x12\x1a\n\x08\x65ndpoint\x18\x01 \x01(\tR\x08\x65ndpoint\x12\x16\n\x05table\x18\x02 \x01(\tH\x00R\x05table\x12\x16\n\x05query\x18\x05 \x01(\tH\x00R\x05query\x12X\n\x14raw_batch_translator\x18\x03 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x12rawBatchTranslator\x12#\n\rtimestamp_key\x18\x06 \x01(\tR\x0ctimestampKeyB\x08\n\x06sourceJ\x04\x08\x04\x10\x05\"\xd1\x02\n\x17SnowflakeDataSourceArgs\x12\x10\n\x03url\x18\x01 \x01(\tR\x03url\x12\x12\n\x04role\x18\x02 \x01(\tR\x04role\x12\x1a\n\x08\x64\x61tabase\x18\x03 \x01(\tR\x08\x64\x61tabase\x12\x16\n\x06schema\x18\x04 \x01(\tR\x06schema\x12#\n\twarehouse\x18\x05 \x01(\tB\x05\x92M\x02\x08\x01R\twarehouse\x12\x16\n\x05table\x18\x06 \x01(\tH\x00R\x05table\x12\x16\n\x05query\x18\x07 \x01(\tH\x00R\x05query\x12X\n\x14raw_batch_translator\x18\x08 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x12rawBatchTranslator\x12#\n\rtimestamp_key\x18\t \x01(\tR\x0ctimestampKeyB\x08\n\x06sourceB\x13\n\x0f\x63om.tecton.argsP\x01'
  ,
  dependencies=[google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_data__source__config__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_diff__options__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_user__defined__function__pb2.DESCRIPTOR,tecton__proto_dot_common_dot_spark__schema__pb2.DESCRIPTOR,])




_DATETIMEPARTITIONCOLUMNARGS = _descriptor.Descriptor(
  name='DatetimePartitionColumnArgs',
  full_name='tecton_proto.args.DatetimePartitionColumnArgs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='column_name', full_name='tecton_proto.args.DatetimePartitionColumnArgs.column_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='columnName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='datepart', full_name='tecton_proto.args.DatetimePartitionColumnArgs.datepart', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='datepart', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='zero_padded', full_name='tecton_proto.args.DatetimePartitionColumnArgs.zero_padded', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='zeroPadded', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=259,
  serialized_end=382,
)


_HIVEDATASOURCEARGS = _descriptor.Descriptor(
  name='HiveDataSourceArgs',
  full_name='tecton_proto.args.HiveDataSourceArgs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='table', full_name='tecton_proto.args.HiveDataSourceArgs.table', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='table', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='database', full_name='tecton_proto.args.HiveDataSourceArgs.database', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='database', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='date_partition_column', full_name='tecton_proto.args.HiveDataSourceArgs.date_partition_column', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='datePartitionColumn', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp_column_name', full_name='tecton_proto.args.HiveDataSourceArgs.timestamp_column_name', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestampColumnName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp_format', full_name='tecton_proto.args.HiveDataSourceArgs.timestamp_format', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestampFormat', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='skip_validation', full_name='tecton_proto.args.HiveDataSourceArgs.skip_validation', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='skipValidation', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='datetime_partition_columns', full_name='tecton_proto.args.HiveDataSourceArgs.datetime_partition_columns', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='datetimePartitionColumns', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='raw_batch_translator', full_name='tecton_proto.args.HiveDataSourceArgs.raw_batch_translator', index=7,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='rawBatchTranslator', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=385,
  serialized_end=849,
)


_FILEDATASOURCEARGS = _descriptor.Descriptor(
  name='FileDataSourceArgs',
  full_name='tecton_proto.args.FileDataSourceArgs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='uri', full_name='tecton_proto.args.FileDataSourceArgs.uri', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='uri', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='file_format', full_name='tecton_proto.args.FileDataSourceArgs.file_format', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='fileFormat', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='convert_to_glue_format', full_name='tecton_proto.args.FileDataSourceArgs.convert_to_glue_format', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='convertToGlueFormat', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='raw_batch_translator', full_name='tecton_proto.args.FileDataSourceArgs.raw_batch_translator', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='rawBatchTranslator', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='schema_uri', full_name='tecton_proto.args.FileDataSourceArgs.schema_uri', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222M\002\010\001', json_name='schemaUri', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp_column_name', full_name='tecton_proto.args.FileDataSourceArgs.timestamp_column_name', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestampColumnName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp_format', full_name='tecton_proto.args.FileDataSourceArgs.timestamp_format', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestampFormat', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='schema_override', full_name='tecton_proto.args.FileDataSourceArgs.schema_override', index=7,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaOverride', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=852,
  serialized_end=1280,
)


_OPTION = _descriptor.Descriptor(
  name='Option',
  full_name='tecton_proto.args.Option',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='tecton_proto.args.Option.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='key', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='tecton_proto.args.Option.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1282,
  serialized_end=1330,
)


_KINESISDATASOURCEARGS = _descriptor.Descriptor(
  name='KinesisDataSourceArgs',
  full_name='tecton_proto.args.KinesisDataSourceArgs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='stream_name', full_name='tecton_proto.args.KinesisDataSourceArgs.stream_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='streamName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='region', full_name='tecton_proto.args.KinesisDataSourceArgs.region', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='region', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='raw_stream_translator', full_name='tecton_proto.args.KinesisDataSourceArgs.raw_stream_translator', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='rawStreamTranslator', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp_key', full_name='tecton_proto.args.KinesisDataSourceArgs.timestamp_key', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestampKey', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='default_initial_stream_position', full_name='tecton_proto.args.KinesisDataSourceArgs.default_initial_stream_position', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='defaultInitialStreamPosition', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='default_watermark_delay_threshold', full_name='tecton_proto.args.KinesisDataSourceArgs.default_watermark_delay_threshold', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='defaultWatermarkDelayThreshold', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deduplication_columns', full_name='tecton_proto.args.KinesisDataSourceArgs.deduplication_columns', index=6,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='deduplicationColumns', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='options', full_name='tecton_proto.args.KinesisDataSourceArgs.options', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='options', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1333,
  serialized_end=1869,
)


_KAFKADATASOURCEARGS = _descriptor.Descriptor(
  name='KafkaDataSourceArgs',
  full_name='tecton_proto.args.KafkaDataSourceArgs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='kafka_bootstrap_servers', full_name='tecton_proto.args.KafkaDataSourceArgs.kafka_bootstrap_servers', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='kafkaBootstrapServers', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='topics', full_name='tecton_proto.args.KafkaDataSourceArgs.topics', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='topics', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='raw_stream_translator', full_name='tecton_proto.args.KafkaDataSourceArgs.raw_stream_translator', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='rawStreamTranslator', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp_key', full_name='tecton_proto.args.KafkaDataSourceArgs.timestamp_key', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestampKey', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='default_watermark_delay_threshold', full_name='tecton_proto.args.KafkaDataSourceArgs.default_watermark_delay_threshold', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='defaultWatermarkDelayThreshold', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='options', full_name='tecton_proto.args.KafkaDataSourceArgs.options', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='options', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ssl_keystore_location', full_name='tecton_proto.args.KafkaDataSourceArgs.ssl_keystore_location', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222M\002\010\001', json_name='sslKeystoreLocation', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ssl_keystore_password_secret_id', full_name='tecton_proto.args.KafkaDataSourceArgs.ssl_keystore_password_secret_id', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222M\002\010\001', json_name='sslKeystorePasswordSecretId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ssl_truststore_location', full_name='tecton_proto.args.KafkaDataSourceArgs.ssl_truststore_location', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222M\002\010\001', json_name='sslTruststoreLocation', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ssl_truststore_password_secret_id', full_name='tecton_proto.args.KafkaDataSourceArgs.ssl_truststore_password_secret_id', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222M\002\010\001', json_name='sslTruststorePasswordSecretId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='security_protocol', full_name='tecton_proto.args.KafkaDataSourceArgs.security_protocol', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222M\002\010\001', json_name='securityProtocol', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1872,
  serialized_end=2589,
)


_REDSHIFTDATASOURCEARGS = _descriptor.Descriptor(
  name='RedshiftDataSourceArgs',
  full_name='tecton_proto.args.RedshiftDataSourceArgs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='endpoint', full_name='tecton_proto.args.RedshiftDataSourceArgs.endpoint', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='endpoint', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='table', full_name='tecton_proto.args.RedshiftDataSourceArgs.table', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='table', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='query', full_name='tecton_proto.args.RedshiftDataSourceArgs.query', index=2,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='query', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='raw_batch_translator', full_name='tecton_proto.args.RedshiftDataSourceArgs.raw_batch_translator', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='rawBatchTranslator', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp_key', full_name='tecton_proto.args.RedshiftDataSourceArgs.timestamp_key', index=4,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestampKey', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='source', full_name='tecton_proto.args.RedshiftDataSourceArgs.source',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=2592,
  serialized_end=2835,
)


_SNOWFLAKEDATASOURCEARGS = _descriptor.Descriptor(
  name='SnowflakeDataSourceArgs',
  full_name='tecton_proto.args.SnowflakeDataSourceArgs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='url', full_name='tecton_proto.args.SnowflakeDataSourceArgs.url', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='url', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='role', full_name='tecton_proto.args.SnowflakeDataSourceArgs.role', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='role', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='database', full_name='tecton_proto.args.SnowflakeDataSourceArgs.database', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='database', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='schema', full_name='tecton_proto.args.SnowflakeDataSourceArgs.schema', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schema', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='warehouse', full_name='tecton_proto.args.SnowflakeDataSourceArgs.warehouse', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222M\002\010\001', json_name='warehouse', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='table', full_name='tecton_proto.args.SnowflakeDataSourceArgs.table', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='table', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='query', full_name='tecton_proto.args.SnowflakeDataSourceArgs.query', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='query', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='raw_batch_translator', full_name='tecton_proto.args.SnowflakeDataSourceArgs.raw_batch_translator', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='rawBatchTranslator', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp_key', full_name='tecton_proto.args.SnowflakeDataSourceArgs.timestamp_key', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestampKey', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='source', full_name='tecton_proto.args.SnowflakeDataSourceArgs.source',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=2838,
  serialized_end=3175,
)

_HIVEDATASOURCEARGS.fields_by_name['datetime_partition_columns'].message_type = _DATETIMEPARTITIONCOLUMNARGS
_HIVEDATASOURCEARGS.fields_by_name['raw_batch_translator'].message_type = tecton__proto_dot_args_dot_user__defined__function__pb2._USERDEFINEDFUNCTION
_FILEDATASOURCEARGS.fields_by_name['raw_batch_translator'].message_type = tecton__proto_dot_args_dot_user__defined__function__pb2._USERDEFINEDFUNCTION
_FILEDATASOURCEARGS.fields_by_name['schema_override'].message_type = tecton__proto_dot_common_dot_spark__schema__pb2._SPARKSCHEMA
_KINESISDATASOURCEARGS.fields_by_name['raw_stream_translator'].message_type = tecton__proto_dot_args_dot_user__defined__function__pb2._USERDEFINEDFUNCTION
_KINESISDATASOURCEARGS.fields_by_name['default_initial_stream_position'].enum_type = tecton__proto_dot_args_dot_data__source__config__pb2._INITIALSTREAMPOSITION
_KINESISDATASOURCEARGS.fields_by_name['default_watermark_delay_threshold'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_KINESISDATASOURCEARGS.fields_by_name['options'].message_type = _OPTION
_KAFKADATASOURCEARGS.fields_by_name['raw_stream_translator'].message_type = tecton__proto_dot_args_dot_user__defined__function__pb2._USERDEFINEDFUNCTION
_KAFKADATASOURCEARGS.fields_by_name['default_watermark_delay_threshold'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_KAFKADATASOURCEARGS.fields_by_name['options'].message_type = _OPTION
_REDSHIFTDATASOURCEARGS.fields_by_name['raw_batch_translator'].message_type = tecton__proto_dot_args_dot_user__defined__function__pb2._USERDEFINEDFUNCTION
_REDSHIFTDATASOURCEARGS.oneofs_by_name['source'].fields.append(
  _REDSHIFTDATASOURCEARGS.fields_by_name['table'])
_REDSHIFTDATASOURCEARGS.fields_by_name['table'].containing_oneof = _REDSHIFTDATASOURCEARGS.oneofs_by_name['source']
_REDSHIFTDATASOURCEARGS.oneofs_by_name['source'].fields.append(
  _REDSHIFTDATASOURCEARGS.fields_by_name['query'])
_REDSHIFTDATASOURCEARGS.fields_by_name['query'].containing_oneof = _REDSHIFTDATASOURCEARGS.oneofs_by_name['source']
_SNOWFLAKEDATASOURCEARGS.fields_by_name['raw_batch_translator'].message_type = tecton__proto_dot_args_dot_user__defined__function__pb2._USERDEFINEDFUNCTION
_SNOWFLAKEDATASOURCEARGS.oneofs_by_name['source'].fields.append(
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['table'])
_SNOWFLAKEDATASOURCEARGS.fields_by_name['table'].containing_oneof = _SNOWFLAKEDATASOURCEARGS.oneofs_by_name['source']
_SNOWFLAKEDATASOURCEARGS.oneofs_by_name['source'].fields.append(
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['query'])
_SNOWFLAKEDATASOURCEARGS.fields_by_name['query'].containing_oneof = _SNOWFLAKEDATASOURCEARGS.oneofs_by_name['source']
DESCRIPTOR.message_types_by_name['DatetimePartitionColumnArgs'] = _DATETIMEPARTITIONCOLUMNARGS
DESCRIPTOR.message_types_by_name['HiveDataSourceArgs'] = _HIVEDATASOURCEARGS
DESCRIPTOR.message_types_by_name['FileDataSourceArgs'] = _FILEDATASOURCEARGS
DESCRIPTOR.message_types_by_name['Option'] = _OPTION
DESCRIPTOR.message_types_by_name['KinesisDataSourceArgs'] = _KINESISDATASOURCEARGS
DESCRIPTOR.message_types_by_name['KafkaDataSourceArgs'] = _KAFKADATASOURCEARGS
DESCRIPTOR.message_types_by_name['RedshiftDataSourceArgs'] = _REDSHIFTDATASOURCEARGS
DESCRIPTOR.message_types_by_name['SnowflakeDataSourceArgs'] = _SNOWFLAKEDATASOURCEARGS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DatetimePartitionColumnArgs = _reflection.GeneratedProtocolMessageType('DatetimePartitionColumnArgs', (_message.Message,), {
  'DESCRIPTOR' : _DATETIMEPARTITIONCOLUMNARGS,
  '__module__' : 'tecton_proto.args.data_source_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.DatetimePartitionColumnArgs)
  })
_sym_db.RegisterMessage(DatetimePartitionColumnArgs)

HiveDataSourceArgs = _reflection.GeneratedProtocolMessageType('HiveDataSourceArgs', (_message.Message,), {
  'DESCRIPTOR' : _HIVEDATASOURCEARGS,
  '__module__' : 'tecton_proto.args.data_source_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.HiveDataSourceArgs)
  })
_sym_db.RegisterMessage(HiveDataSourceArgs)

FileDataSourceArgs = _reflection.GeneratedProtocolMessageType('FileDataSourceArgs', (_message.Message,), {
  'DESCRIPTOR' : _FILEDATASOURCEARGS,
  '__module__' : 'tecton_proto.args.data_source_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.FileDataSourceArgs)
  })
_sym_db.RegisterMessage(FileDataSourceArgs)

Option = _reflection.GeneratedProtocolMessageType('Option', (_message.Message,), {
  'DESCRIPTOR' : _OPTION,
  '__module__' : 'tecton_proto.args.data_source_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.Option)
  })
_sym_db.RegisterMessage(Option)

KinesisDataSourceArgs = _reflection.GeneratedProtocolMessageType('KinesisDataSourceArgs', (_message.Message,), {
  'DESCRIPTOR' : _KINESISDATASOURCEARGS,
  '__module__' : 'tecton_proto.args.data_source_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.KinesisDataSourceArgs)
  })
_sym_db.RegisterMessage(KinesisDataSourceArgs)

KafkaDataSourceArgs = _reflection.GeneratedProtocolMessageType('KafkaDataSourceArgs', (_message.Message,), {
  'DESCRIPTOR' : _KAFKADATASOURCEARGS,
  '__module__' : 'tecton_proto.args.data_source_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.KafkaDataSourceArgs)
  })
_sym_db.RegisterMessage(KafkaDataSourceArgs)

RedshiftDataSourceArgs = _reflection.GeneratedProtocolMessageType('RedshiftDataSourceArgs', (_message.Message,), {
  'DESCRIPTOR' : _REDSHIFTDATASOURCEARGS,
  '__module__' : 'tecton_proto.args.data_source_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.RedshiftDataSourceArgs)
  })
_sym_db.RegisterMessage(RedshiftDataSourceArgs)

SnowflakeDataSourceArgs = _reflection.GeneratedProtocolMessageType('SnowflakeDataSourceArgs', (_message.Message,), {
  'DESCRIPTOR' : _SNOWFLAKEDATASOURCEARGS,
  '__module__' : 'tecton_proto.args.data_source_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.SnowflakeDataSourceArgs)
  })
_sym_db.RegisterMessage(SnowflakeDataSourceArgs)


DESCRIPTOR._options = None
_FILEDATASOURCEARGS.fields_by_name['schema_uri']._options = None
_KAFKADATASOURCEARGS.fields_by_name['ssl_keystore_location']._options = None
_KAFKADATASOURCEARGS.fields_by_name['ssl_keystore_password_secret_id']._options = None
_KAFKADATASOURCEARGS.fields_by_name['ssl_truststore_location']._options = None
_KAFKADATASOURCEARGS.fields_by_name['ssl_truststore_password_secret_id']._options = None
_KAFKADATASOURCEARGS.fields_by_name['security_protocol']._options = None
_SNOWFLAKEDATASOURCEARGS.fields_by_name['warehouse']._options = None
# @@protoc_insertion_point(module_scope)



import datetime
from typing import List

import pendulum

from tecton_spark import time_utils
from tecton_spark.feature_definition_wrapper import FeatureDefinitionWrapper as FeatureDefinition
from tecton_spark.spark_time_utils import convert_epoch_to_datetime
from tecton_spark.spark_time_utils import convert_timestamp_to_epoch
from tecton_spark.spark_time_utils import subtract_seconds_from_timestamp
from tecton_spark.time_utils import convert_timedelta_for_version

DIAGNOSTIC_DATE_FORMAT = "%Y-%m-%d %H:%M:%S %Z"


class MaterializationParams(object):
    """
    Holds the configuration for scheduled materializations.
    """

    def __init__(
        self,
        min_scheduling_interval: pendulum.Duration,
        allowed_upstream_lateness: pendulum.Duration,
        from_timestamp: pendulum.datetime,
        batch_schedule: pendulum.Duration = pendulum.Duration(),
        data_partitions_coalesce_override: int = None,
        feature_definition: FeatureDefinition = None,
        **kwargs,
    ):
        """
        :param min_scheduling_interval: Minimum scheduling time interval.
        :param batch_schedule: Batch materialization schedule
        :param allowed_upstream_lateness: The materialization run that
            consumes an event with timestamp T will not run until after
            this time after T.
        :param from_timestamp: No materialization runs will be scheduled with
            an anchor before this time.
        :param data_partitions_coalesce_override: Override the number of partitions in the materialized data (on top of tiles).
        """
        self.feature_definition = feature_definition
        self.min_scheduling_interval = time_utils.assert_is_round_seconds(min_scheduling_interval)
        self.batch_schedule = time_utils.assert_is_round_seconds(batch_schedule)
        self.allowed_upstream_lateness = time_utils.assert_is_round_seconds(allowed_upstream_lateness)
        self.from_timestamp = from_timestamp
        self.data_partitions_coalesce_override = data_partitions_coalesce_override

    @staticmethod
    def from_feature_definition(feature_definition: FeatureDefinition):
        return MaterializationParams(
            feature_definition=feature_definition,
            min_scheduling_interval=feature_definition.min_scheduling_interval,
            batch_schedule=feature_definition.batch_materialization_schedule,
            allowed_upstream_lateness=feature_definition.allowed_upstream_lateness,
            from_timestamp=feature_definition.start_timestamp,
            data_partitions_coalesce_override=feature_definition.data_partitions_coalesce_override,
        )

    def __str__(self):
        lines = [
            f"Batch Materialization Schedule: {self.batch_schedule.in_words()}",
            f"Start Time: {self.from_timestamp.strftime(DIAGNOSTIC_DATE_FORMAT)}",
            f"Schedule Offset: {self.allowed_upstream_lateness_words()}",
        ]
        return "\n".join(lines)

    def allowed_upstream_lateness_words(self):
        in_words = self.allowed_upstream_lateness.in_words()
        # in_words() returns '' for a duration of zero
        return in_words if in_words != "" else "0 seconds"

    def get_tile_interval(self):
        return time_utils.assert_is_round_seconds(self.feature_definition.get_tile_interval)

    def most_recent_anchor(self, timestamp: pendulum.datetime) -> pendulum.datetime:
        """Computes the most recent anchor time which is ready to be computed.

        :param timestamp: The timestamp.
        :return: The timestamp of the greatest ready anchor time <= timestamp.
        """
        anchor_time = self.most_recent_anchor_time(timestamp)
        return convert_epoch_to_datetime(anchor_time, self.feature_definition.get_feature_store_format_version)

    def most_recent_anchor_time(self, timestamp: datetime.datetime) -> int:
        """Computes the most recent anchor time which is ready to be computed.

        :param timestamp: The timestamp in python date time
        :return: The timestamp in seconds of the greatest ready anchor time <= timestamp.
        """
        return self.most_recent_tile_end_time(
            subtract_seconds_from_timestamp(timestamp, self.min_scheduling_interval.in_seconds())
        )

    def most_recent_tile_end_time(self, timestamp: datetime.datetime) -> int:
        """Computes the most recent tile end time which is ready to be computed.

        :param timestamp: The timestamp in python datetime format
        :return: The timestamp in seconds of the greatest ready tile end time <= timestamp.
        """
        return self.align_timestamp_left(
            subtract_seconds_from_timestamp(timestamp, self.allowed_upstream_lateness.in_seconds())
        )

    def align(self, timestamp: pendulum.datetime) -> pendulum.datetime:
        """Aligns a time to this schedule.

        This function does not take allowed lateness into account.

        :param timestamp: The timestamp to align.
        :return: The timestamp of the greatest aligned time <= timestamp.
        """
        aligned_time = self.align_timestamp_left(timestamp)
        return convert_epoch_to_datetime(aligned_time, self.feature_definition.get_feature_store_format_version)

    def align_timestamp_left(self, timestamp: datetime.datetime) -> int:
        """Aligns a time to this schedule.

        This function does not take allowed lateness into account.

        :param timestamp: The timestamp in python datetime format
        :return: The timestamp of the greatest aligned time <= timestamp, in seconds.
        """
        is_continuous_aggregate = (
            self.feature_definition.is_temporal_aggregate and self.feature_definition.is_continuous_temporal_aggregate
        )
        if not self.min_scheduling_interval and not is_continuous_aggregate:
            raise Exception(f"Interval must be greater than 0; got {self.min_scheduling_interval}")

        version = self.feature_definition.get_feature_store_format_version
        scheduling_interval = convert_timedelta_for_version(self.min_scheduling_interval, version)
        timestamp_epoch = convert_timestamp_to_epoch(timestamp, version)
        aligning_delta = 0
        if scheduling_interval > 0:
            aligning_delta = timestamp_epoch % scheduling_interval

        return timestamp_epoch - aligning_delta

    def force_align_timestamp_right(self, timestamp: datetime.datetime) -> int:
        """Aligns timestamp to the scheduling interval. If it's already aligned,
        adds scheduling interval's length.

        This function does not take allowed lateness into account.

        :param timestamp: The timestamp in python datetime format
        :return: The timestamp of the greatest aligned time <= timestamp, in seconds.
        """
        aligned_left = self.align_timestamp_left(timestamp)

        version = self.feature_definition.get_feature_store_format_version
        scheduling_interval = convert_timedelta_for_version(self.min_scheduling_interval, version)

        return aligned_left + scheduling_interval

    def align_timestamp_right(self, timestamp: datetime.datetime) -> int:
        """Aligns a time to this schedule.

        :param timestamp: The timestamp in python datetime format
        :return: The timestamp of the lowest aligned time >= timestamp, in seconds.
        """
        is_continuous_aggregate = (
            self.feature_definition.is_temporal_aggregate and self.feature_definition.is_continuous_temporal_aggregate
        )
        if not self.min_scheduling_interval and not is_continuous_aggregate:
            raise Exception(f"Interval must be greater than 0; got {self.min_scheduling_interval}")

        version = self.feature_definition.get_feature_store_format_version
        scheduling_interval = convert_timedelta_for_version(self.min_scheduling_interval, version)
        timestamp_epoch = convert_timestamp_to_epoch(timestamp, version)
        aligning_delta = 0
        if scheduling_interval > 0:
            aligning_delta = timestamp_epoch % scheduling_interval

        timestamp_epoch = timestamp_epoch + scheduling_interval - aligning_delta
        return timestamp_epoch

    def construct_anchor_times(self, start_time, num_tiles, version) -> List[int]:
        """Creates `num_tiles` consecutive anchor_times starting from `start_time`.

        :return: An increasing list of consecutive anchor times.
        """
        anchor_times = []
        for i in range(num_tiles):
            anchor_time = start_time + i * self.get_tile_interval()
            anchor_time_val = convert_timestamp_to_epoch(anchor_time, version)
            anchor_times.append(anchor_time_val)

        return anchor_times

    @staticmethod
    def time_range(start_time, end_time) -> pendulum.Period:
        return pendulum.instance(end_time) - pendulum.instance(start_time)

    def construct_tile_end_times(self, latest_tile_end_time, num_tiles, version) -> List[int]:
        """Creates `num_tiles` consecutive tile_end_times where latest one ends at `latest_tile_end_time`.

        :return: An increasing list of consecutive tile end times.
        """
        tile_end_times = []
        for i in range(num_tiles):
            tile_end_time = latest_tile_end_time - i * self.batch_schedule
            time_val = convert_timestamp_to_epoch(tile_end_time, version)
            tile_end_times.append(time_val)

        tile_end_times.reverse()
        return tile_end_times



from typing import Dict
from typing import List
from typing import Optional

from tecton.data_sources.base_data_source_config import BaseStreamDSConfig
from tecton_proto.args import data_source_pb2
from tecton_proto.args import virtual_data_source_pb2
from tecton_spark import data_source_helper
from tecton_spark import function_serialization
from tecton_spark.time_utils import strict_pytimeparse


class KinesisDSConfig(BaseStreamDSConfig):
    """
    Configuration used to reference a Kinesis stream.

    The KinesisDSConfig class is used to create a reference to an AWS Kinesis stream.

    This class used as an input to a :class:`StreamDataSource`'s parameter ``stream_ds_config``. This class is not
    a Tecton Object: it is a grouping of parameters. Declaring this class alone will not register a data source.
    Instead, declare as part of ``StreamDataSource`` that takes this configuration class instance as a parameter.
    """

    def __init__(
        self,
        stream_name: str,
        region: str,
        raw_stream_translator,
        timestamp_key: str,
        default_initial_stream_position: str,
        default_watermark_delay_threshold: str,
        deduplication_columns: List[str] = None,
        options: Optional[Dict[str, str]] = None,
    ):
        """
        Instantiates a new KinesisDSConfig.

        :param stream_name: Name of the Kinesis stream.
        :param region: AWS region of the stream, e.g: "us-west-2".
        :param raw_stream_translator: Python user defined function f(DataFrame) -> DataFrame that takes in raw
                                      Pyspark data source DataFrame and translates it to the DataFrame to be
                                      consumed by the Feature View. See an example of
                                      raw_stream_translator in the `User Guide`_.
        :param timestamp_key: Name of the column containing timestamp for watermarking.
        :param default_initial_stream_position: Initial position in stream, e.g: "latest" or "trim_horizon".
                                                More information available in `Spark Kinesis Documentation`_.
        :param default_watermark_delay_threshold: Watermark time interval, e.g: "24 hours", used by Spark Structured Streaming to account for late-arriving data. See: https://docs.tecton.ai/v2/overviews/framework/feature_views/stream_feature_view.html#productionizing-a-stream
        :param deduplication_columns: (Optional) Columns in the stream data that uniquely identify data records.
                                        Used for de-duplicating.
        :param options: (Optional) A map of additional Spark readStream options

        :return: A KinesisDSConfig class instance.

        .. _User Guide: https://docs.tecton.ai/v2/overviews/framework/data_sources.html
        .. _Spark Kinesis Documentation: https://spark.apache.org/docs/latest/streaming-kinesis-integration.html

        Example of a KinesisDSConfig declaration:

        .. code-block:: python

            import pyspark
            from tecton import KinesisDSConfig


            # Define our deserialization raw stream translator
            def raw_data_deserialization(df:pyspark.sql.DataFrame) -> pyspark.sql.DataFrame:
                from pyspark.sql.functions import col, from_json, from_utc_timestamp
                from pyspark.sql.types import StructType, StringType

                payload_schema = (
                  StructType()
                        .add('amount', StringType(), False)
                        .add('isFraud', StringType(), False)
                        .add('timestamp', StringType(), False)
                )

                return (
                    df.selectExpr('cast (data as STRING) jsonData')
                    .select(from_json('jsonData', payload_schema).alias('payload'))
                    .select(
                        col('payload.amount').cast('long').alias('amount'),
                        col('payload.isFraud').cast('long').alias('isFraud'),
                        from_utc_timestamp('payload.timestamp', 'UTC').alias('timestamp')
                    )
                )
            # Declare KinesisDSConfig instance object that can be used as argument in `StreamDataSource`
            stream_ds_config = KinesisDSConfig(
                                    stream_name='transaction_events',
                                    region='us-west-2',
                                    default_initial_stream_position='latest',
                                    default_watermark_delay_threshold='30 minutes',
                                    timestamp_key='timestamp',
                                    raw_stream_translator=raw_data_deserialization,
                                    options={'roleArn': 'arn:aws:iam::472542229217:role/demo-cross-account-kinesis-ro'}
            )
        """

        self._args = prepare_kinesis_ds_args(
            stream_name=stream_name,
            region=region,
            raw_stream_translator=raw_stream_translator,
            timestamp_key=timestamp_key,
            default_initial_stream_position=default_initial_stream_position,
            default_watermark_delay_threshold=default_watermark_delay_threshold,
            deduplication_columns=deduplication_columns,
            options=options,
        )

    def _merge_stream_args(self, data_source_args: virtual_data_source_pb2.VirtualDataSourceArgs):
        data_source_args.kinesis_ds_config.CopyFrom(self._args)


def prepare_kinesis_ds_args(
    *,
    stream_name: str,
    region: str,
    raw_stream_translator,
    timestamp_key: str,
    default_initial_stream_position: Optional[str],
    default_watermark_delay_threshold: Optional[str],
    deduplication_columns: Optional[List[str]],
    options: Optional[Dict[str, str]],
):
    args = data_source_pb2.KinesisDataSourceArgs()
    args.stream_name = stream_name
    args.region = region
    args.raw_stream_translator.CopyFrom(function_serialization.to_proto(raw_stream_translator))
    args.timestamp_key = timestamp_key
    if default_initial_stream_position:
        args.default_initial_stream_position = data_source_helper.INITIAL_STREAM_POSITION_STR_TO_ENUM[
            default_initial_stream_position
        ]
    if default_watermark_delay_threshold:
        args.default_watermark_delay_threshold.FromSeconds(strict_pytimeparse(default_watermark_delay_threshold))
    if deduplication_columns:
        for column_name in deduplication_columns:
            args.deduplication_columns.append(column_name)
    options_ = options or {}
    for key in sorted(options_.keys()):
        option = data_source_pb2.Option()
        option.key = key
        option.value = options_[key]
        args.options.append(option)

    return args



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/virtual_data_source.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.args import basic_info_pb2 as tecton__proto_dot_args_dot_basic__info__pb2
from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2
from tecton_proto.args import data_source_pb2 as tecton__proto_dot_args_dot_data__source__pb2
from tecton_proto.args import diff_options_pb2 as tecton__proto_dot_args_dot_diff__options__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/args/virtual_data_source.proto',
  package='tecton_proto.args',
  syntax='proto2',
  serialized_options=b'\n\017com.tecton.argsP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n+tecton_proto/args/virtual_data_source.proto\x12\x11tecton_proto.args\x1a\"tecton_proto/args/basic_info.proto\x1a\x1ctecton_proto/common/id.proto\x1a#tecton_proto/args/data_source.proto\x1a$tecton_proto/args/diff_options.proto\"\x84\x06\n\x15VirtualDataSourceArgs\x12L\n\x16virtual_data_source_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x13virtualDataSourceId\x12\x37\n\x04info\x18\x02 \x01(\x0b\x32\x1c.tecton_proto.args.BasicInfoB\x05\x92M\x02\x10\x01R\x04info\x12M\n\x0ehive_ds_config\x18\x03 \x01(\x0b\x32%.tecton_proto.args.HiveDataSourceArgsH\x00R\x0chiveDsConfig\x12M\n\x0e\x66ile_ds_config\x18\x04 \x01(\x0b\x32%.tecton_proto.args.FileDataSourceArgsH\x00R\x0c\x66ileDsConfig\x12Y\n\x12redshift_ds_config\x18\x07 \x01(\x0b\x32).tecton_proto.args.RedshiftDataSourceArgsH\x00R\x10redshiftDsConfig\x12\\\n\x13snowflake_ds_config\x18\x08 \x01(\x0b\x32*.tecton_proto.args.SnowflakeDataSourceArgsH\x00R\x11snowflakeDsConfig\x12V\n\x11kinesis_ds_config\x18\x05 \x01(\x0b\x32(.tecton_proto.args.KinesisDataSourceArgsH\x01R\x0fkinesisDsConfig\x12P\n\x0fkafka_ds_config\x18\x06 \x01(\x0b\x32&.tecton_proto.args.KafkaDataSourceArgsH\x01R\rkafkaDsConfig\x12<\n\x04type\x18\t \x01(\x0e\x32!.tecton_proto.args.DataSourceTypeB\x05\x92M\x02\x08\x03R\x04typeB\x11\n\x0f\x62\x61tch_ds_configB\x12\n\x10stream_ds_config*?\n\x0e\x44\x61taSourceType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\t\n\x05\x42\x41TCH\x10\x01\x12\x15\n\x11STREAM_WITH_BATCH\x10\x02\x42\x13\n\x0f\x63om.tecton.argsP\x01'
  ,
  dependencies=[tecton__proto_dot_args_dot_basic__info__pb2.DESCRIPTOR,tecton__proto_dot_common_dot_id__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_data__source__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_diff__options__pb2.DESCRIPTOR,])

_DATASOURCETYPE = _descriptor.EnumDescriptor(
  name='DataSourceType',
  full_name='tecton_proto.args.DataSourceType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BATCH', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='STREAM_WITH_BATCH', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=982,
  serialized_end=1045,
)
_sym_db.RegisterEnumDescriptor(_DATASOURCETYPE)

DataSourceType = enum_type_wrapper.EnumTypeWrapper(_DATASOURCETYPE)
UNKNOWN = 0
BATCH = 1
STREAM_WITH_BATCH = 2



_VIRTUALDATASOURCEARGS = _descriptor.Descriptor(
  name='VirtualDataSourceArgs',
  full_name='tecton_proto.args.VirtualDataSourceArgs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='virtual_data_source_id', full_name='tecton_proto.args.VirtualDataSourceArgs.virtual_data_source_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='virtualDataSourceId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='info', full_name='tecton_proto.args.VirtualDataSourceArgs.info', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222M\002\020\001', json_name='info', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hive_ds_config', full_name='tecton_proto.args.VirtualDataSourceArgs.hive_ds_config', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='hiveDsConfig', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='file_ds_config', full_name='tecton_proto.args.VirtualDataSourceArgs.file_ds_config', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='fileDsConfig', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='redshift_ds_config', full_name='tecton_proto.args.VirtualDataSourceArgs.redshift_ds_config', index=4,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='redshiftDsConfig', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='snowflake_ds_config', full_name='tecton_proto.args.VirtualDataSourceArgs.snowflake_ds_config', index=5,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='snowflakeDsConfig', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='kinesis_ds_config', full_name='tecton_proto.args.VirtualDataSourceArgs.kinesis_ds_config', index=6,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='kinesisDsConfig', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='kafka_ds_config', full_name='tecton_proto.args.VirtualDataSourceArgs.kafka_ds_config', index=7,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='kafkaDsConfig', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='tecton_proto.args.VirtualDataSourceArgs.type', index=8,
      number=9, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222M\002\010\003', json_name='type', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='batch_ds_config', full_name='tecton_proto.args.VirtualDataSourceArgs.batch_ds_config',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='stream_ds_config', full_name='tecton_proto.args.VirtualDataSourceArgs.stream_ds_config',
      index=1, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=208,
  serialized_end=980,
)

_VIRTUALDATASOURCEARGS.fields_by_name['virtual_data_source_id'].message_type = tecton__proto_dot_common_dot_id__pb2._ID
_VIRTUALDATASOURCEARGS.fields_by_name['info'].message_type = tecton__proto_dot_args_dot_basic__info__pb2._BASICINFO
_VIRTUALDATASOURCEARGS.fields_by_name['hive_ds_config'].message_type = tecton__proto_dot_args_dot_data__source__pb2._HIVEDATASOURCEARGS
_VIRTUALDATASOURCEARGS.fields_by_name['file_ds_config'].message_type = tecton__proto_dot_args_dot_data__source__pb2._FILEDATASOURCEARGS
_VIRTUALDATASOURCEARGS.fields_by_name['redshift_ds_config'].message_type = tecton__proto_dot_args_dot_data__source__pb2._REDSHIFTDATASOURCEARGS
_VIRTUALDATASOURCEARGS.fields_by_name['snowflake_ds_config'].message_type = tecton__proto_dot_args_dot_data__source__pb2._SNOWFLAKEDATASOURCEARGS
_VIRTUALDATASOURCEARGS.fields_by_name['kinesis_ds_config'].message_type = tecton__proto_dot_args_dot_data__source__pb2._KINESISDATASOURCEARGS
_VIRTUALDATASOURCEARGS.fields_by_name['kafka_ds_config'].message_type = tecton__proto_dot_args_dot_data__source__pb2._KAFKADATASOURCEARGS
_VIRTUALDATASOURCEARGS.fields_by_name['type'].enum_type = _DATASOURCETYPE
_VIRTUALDATASOURCEARGS.oneofs_by_name['batch_ds_config'].fields.append(
  _VIRTUALDATASOURCEARGS.fields_by_name['hive_ds_config'])
_VIRTUALDATASOURCEARGS.fields_by_name['hive_ds_config'].containing_oneof = _VIRTUALDATASOURCEARGS.oneofs_by_name['batch_ds_config']
_VIRTUALDATASOURCEARGS.oneofs_by_name['batch_ds_config'].fields.append(
  _VIRTUALDATASOURCEARGS.fields_by_name['file_ds_config'])
_VIRTUALDATASOURCEARGS.fields_by_name['file_ds_config'].containing_oneof = _VIRTUALDATASOURCEARGS.oneofs_by_name['batch_ds_config']
_VIRTUALDATASOURCEARGS.oneofs_by_name['batch_ds_config'].fields.append(
  _VIRTUALDATASOURCEARGS.fields_by_name['redshift_ds_config'])
_VIRTUALDATASOURCEARGS.fields_by_name['redshift_ds_config'].containing_oneof = _VIRTUALDATASOURCEARGS.oneofs_by_name['batch_ds_config']
_VIRTUALDATASOURCEARGS.oneofs_by_name['batch_ds_config'].fields.append(
  _VIRTUALDATASOURCEARGS.fields_by_name['snowflake_ds_config'])
_VIRTUALDATASOURCEARGS.fields_by_name['snowflake_ds_config'].containing_oneof = _VIRTUALDATASOURCEARGS.oneofs_by_name['batch_ds_config']
_VIRTUALDATASOURCEARGS.oneofs_by_name['stream_ds_config'].fields.append(
  _VIRTUALDATASOURCEARGS.fields_by_name['kinesis_ds_config'])
_VIRTUALDATASOURCEARGS.fields_by_name['kinesis_ds_config'].containing_oneof = _VIRTUALDATASOURCEARGS.oneofs_by_name['stream_ds_config']
_VIRTUALDATASOURCEARGS.oneofs_by_name['stream_ds_config'].fields.append(
  _VIRTUALDATASOURCEARGS.fields_by_name['kafka_ds_config'])
_VIRTUALDATASOURCEARGS.fields_by_name['kafka_ds_config'].containing_oneof = _VIRTUALDATASOURCEARGS.oneofs_by_name['stream_ds_config']
DESCRIPTOR.message_types_by_name['VirtualDataSourceArgs'] = _VIRTUALDATASOURCEARGS
DESCRIPTOR.enum_types_by_name['DataSourceType'] = _DATASOURCETYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

VirtualDataSourceArgs = _reflection.GeneratedProtocolMessageType('VirtualDataSourceArgs', (_message.Message,), {
  'DESCRIPTOR' : _VIRTUALDATASOURCEARGS,
  '__module__' : 'tecton_proto.args.virtual_data_source_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.VirtualDataSourceArgs)
  })
_sym_db.RegisterMessage(VirtualDataSourceArgs)


DESCRIPTOR._options = None
_VIRTUALDATASOURCEARGS.fields_by_name['info']._options = None
_VIRTUALDATASOURCEARGS.fields_by_name['type']._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/fco_args.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.args import entity_pb2 as tecton__proto_dot_args_dot_entity__pb2
from tecton_proto.args import virtual_data_source_pb2 as tecton__proto_dot_args_dot_virtual__data__source__pb2
from tecton_proto.args import feature_view_pb2 as tecton__proto_dot_args_dot_feature__view__pb2
from tecton_proto.args import feature_service_pb2 as tecton__proto_dot_args_dot_feature__service__pb2
from tecton_proto.args import new_transformation_pb2 as tecton__proto_dot_args_dot_new__transformation__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/args/fco_args.proto',
  package='tecton_proto.args',
  syntax='proto2',
  serialized_options=b'\n\017com.tecton.argsP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n tecton_proto/args/fco_args.proto\x12\x11tecton_proto.args\x1a\x1etecton_proto/args/entity.proto\x1a+tecton_proto/args/virtual_data_source.proto\x1a$tecton_proto/args/feature_view.proto\x1a\'tecton_proto/args/feature_service.proto\x1a*tecton_proto/args/new_transformation.proto\"\xa8\x03\n\x07\x46\x63oArgs\x12Z\n\x13virtual_data_source\x18\x01 \x01(\x0b\x32(.tecton_proto.args.VirtualDataSourceArgsH\x00R\x11virtualDataSource\x12\x37\n\x06\x65ntity\x18\x02 \x01(\x0b\x32\x1d.tecton_proto.args.EntityArgsH\x00R\x06\x65ntity\x12G\n\x0c\x66\x65\x61ture_view\x18\x06 \x01(\x0b\x32\".tecton_proto.args.FeatureViewArgsH\x00R\x0b\x66\x65\x61tureView\x12P\n\x0f\x66\x65\x61ture_service\x18\x04 \x01(\x0b\x32%.tecton_proto.args.FeatureServiceArgsH\x00R\x0e\x66\x65\x61tureService\x12Y\n\x12new_transformation\x18\x07 \x01(\x0b\x32(.tecton_proto.args.NewTransformationArgsH\x00R\x11newTransformationB\x06\n\x04\x61rgsJ\x04\x08\x03\x10\x04J\x04\x08\x05\x10\x06\x42\x13\n\x0f\x63om.tecton.argsP\x01'
  ,
  dependencies=[tecton__proto_dot_args_dot_entity__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_virtual__data__source__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_feature__view__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_feature__service__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_new__transformation__pb2.DESCRIPTOR,])




_FCOARGS = _descriptor.Descriptor(
  name='FcoArgs',
  full_name='tecton_proto.args.FcoArgs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='virtual_data_source', full_name='tecton_proto.args.FcoArgs.virtual_data_source', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='virtualDataSource', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='entity', full_name='tecton_proto.args.FcoArgs.entity', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='entity', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_view', full_name='tecton_proto.args.FcoArgs.feature_view', index=2,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureView', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_service', full_name='tecton_proto.args.FcoArgs.feature_service', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureService', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='new_transformation', full_name='tecton_proto.args.FcoArgs.new_transformation', index=4,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='newTransformation', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='args', full_name='tecton_proto.args.FcoArgs.args',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=256,
  serialized_end=680,
)

_FCOARGS.fields_by_name['virtual_data_source'].message_type = tecton__proto_dot_args_dot_virtual__data__source__pb2._VIRTUALDATASOURCEARGS
_FCOARGS.fields_by_name['entity'].message_type = tecton__proto_dot_args_dot_entity__pb2._ENTITYARGS
_FCOARGS.fields_by_name['feature_view'].message_type = tecton__proto_dot_args_dot_feature__view__pb2._FEATUREVIEWARGS
_FCOARGS.fields_by_name['feature_service'].message_type = tecton__proto_dot_args_dot_feature__service__pb2._FEATURESERVICEARGS
_FCOARGS.fields_by_name['new_transformation'].message_type = tecton__proto_dot_args_dot_new__transformation__pb2._NEWTRANSFORMATIONARGS
_FCOARGS.oneofs_by_name['args'].fields.append(
  _FCOARGS.fields_by_name['virtual_data_source'])
_FCOARGS.fields_by_name['virtual_data_source'].containing_oneof = _FCOARGS.oneofs_by_name['args']
_FCOARGS.oneofs_by_name['args'].fields.append(
  _FCOARGS.fields_by_name['entity'])
_FCOARGS.fields_by_name['entity'].containing_oneof = _FCOARGS.oneofs_by_name['args']
_FCOARGS.oneofs_by_name['args'].fields.append(
  _FCOARGS.fields_by_name['feature_view'])
_FCOARGS.fields_by_name['feature_view'].containing_oneof = _FCOARGS.oneofs_by_name['args']
_FCOARGS.oneofs_by_name['args'].fields.append(
  _FCOARGS.fields_by_name['feature_service'])
_FCOARGS.fields_by_name['feature_service'].containing_oneof = _FCOARGS.oneofs_by_name['args']
_FCOARGS.oneofs_by_name['args'].fields.append(
  _FCOARGS.fields_by_name['new_transformation'])
_FCOARGS.fields_by_name['new_transformation'].containing_oneof = _FCOARGS.oneofs_by_name['args']
DESCRIPTOR.message_types_by_name['FcoArgs'] = _FCOARGS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FcoArgs = _reflection.GeneratedProtocolMessageType('FcoArgs', (_message.Message,), {
  'DESCRIPTOR' : _FCOARGS,
  '__module__' : 'tecton_proto.args.fco_args_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.args.FcoArgs)
  })
_sym_db.RegisterMessage(FcoArgs)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



from datetime import datetime
from typing import Dict
from typing import Optional
from typing import Union

import pandas
import pendulum

from tecton.interactive.data_frame import DataFrame
from tecton.interactive.run_api import validate_and_get_aggregation_level
from tecton.snowflake_context import SnowflakeContext
from tecton_spark import conf
from tecton_spark.feature_definition_wrapper import FeatureDefinitionWrapper as FeatureDefinition
from tecton_spark.feature_set_config import FeatureSetConfig
from tecton_spark.snowflake import sql_helper
from tecton_spark.snowflake.errors import TectonSnowflakeNotImplementedError


def get_historical_features(
    feature_set_config: FeatureSetConfig,
    spine: Optional[Union["snowflake.snowpark.DataFrame", pandas.DataFrame, DataFrame, str]] = None,
    timestamp_key: Optional[str] = None,
    include_feature_view_timestamp_columns: bool = False,
    from_source: bool = False,
    save: bool = False,
    save_as: Optional[str] = None,
    start_time: Optional[Union[pendulum.DateTime, datetime]] = None,
    end_time: Optional[Union[pendulum.DateTime, datetime]] = None,
    entities: Optional[Union["snowflake.snowpark.DataFrame", pandas.DataFrame, DataFrame]] = None,
) -> DataFrame:
    if spine is None:
        raise TectonSnowflakeNotImplementedError("spine is required for Snowflake")

    if timestamp_key is None:
        raise TectonSnowflakeNotImplementedError("timestamp_key must be specified with Snowflake")

    # TODO(TEC-6991): Dataset doesn't really work with snowflake as it has spark dependency.
    # Need to rewrite it with snowflake context or remove this param for snowflake.
    if save or save_as is not None:
        raise TectonSnowflakeNotImplementedError("save is not supported for Snowflake")

    # TODO(TEC-6996): Implement this
    if entities is not None:
        raise TectonSnowflakeNotImplementedError("entities is not supported for Snowflake")

    # TODO(TEC-7010): Implement this
    if start_time is not None or end_time is not None:
        raise TectonSnowflakeNotImplementedError("start_time and end_time are not supported for Snowflake")

    if conf.get_or_none("ALPHA_SNOWFLAKE_SNOWPARK_ENABLED"):
        return DataFrame._create_with_snowflake(
            sql_helper.get_historical_features_with_snowpark(
                spine=spine,
                session=SnowflakeContext.get_instance().get_session(),
                timestamp_key=timestamp_key,
                feature_set_config=feature_set_config,
                include_feature_view_timestamp_columns=include_feature_view_timestamp_columns,
            )
        )
    else:
        return DataFrame._create(
            sql_helper.get_historical_features(
                spine=spine,
                connection=SnowflakeContext.get_instance().get_connection(),
                timestamp_key=timestamp_key,
                feature_set_config=feature_set_config,
                include_feature_view_timestamp_columns=include_feature_view_timestamp_columns,
            )
        )


def run_batch(
    fd: FeatureDefinition,
    mock_inputs: Dict[str, Union[pandas.DataFrame, DataFrame]],
    feature_start_time: Union[pendulum.DateTime, datetime],
    feature_end_time: Union[pendulum.DateTime, datetime],
    aggregate_tiles: bool,
    aggregation_level: str,
) -> DataFrame:
    # TODO(TEC-7053): Implement this
    if len(mock_inputs) > 0:
        raise TectonSnowflakeNotImplementedError("mock_inputs is not supported for Snowflake")

    fv_proto = fd.feature_view_proto
    aggregation_level = validate_and_get_aggregation_level(fd, aggregate_tiles, aggregation_level)

    if fv_proto.HasField("temporal_aggregate"):
        for feature in fv_proto.temporal_aggregate.features:
            aggregate_function = sql_helper.AGGREGATION_PLANS[feature.function]
            if not aggregate_function:
                raise TectonSnowflakeNotImplementedError(
                    f"Unsupported aggregation function {feature.function} in snowflake pipeline"
                )
    sql_str = sql_helper.generate_run_batch_sql(
        feature_view=fv_proto,
        feature_start_time=feature_start_time,
        feature_end_time=feature_end_time,
        aggregation_level=aggregation_level,
    )
    if conf.get_or_none("ALPHA_SNOWFLAKE_SNOWPARK_ENABLED"):
        session = SnowflakeContext.get_instance().get_session()
        return DataFrame._create_with_snowflake(session.sql(sql_str))
    else:
        connection = SnowflakeContext.get_instance().get_connection()
        cur = connection.cursor()
        cur.execute(sql_str)
        return DataFrame._create(cur.fetch_pandas_all())



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/feature_store.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/data/feature_store.proto',
  package='tecton_proto.data',
  syntax='proto2',
  serialized_options=b'\n\017com.tecton.dataP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n%tecton_proto/data/feature_store.proto\x12\x11tecton_proto.data*\xce\x01\n\x19\x46\x65\x61tureStoreFormatVersion\x12(\n$FEATURE_STORE_FORMAT_VERSION_DEFAULT\x10\x00\x12\x31\n-FEATURE_STORE_FORMAT_VERSION_TIME_NANOSECONDS\x10\x01\x12*\n&FEATURE_STORE_FORMAT_VERSION_TTL_FIELD\x10\x02\x12$\n FEATURE_STORE_FORMAT_VERSION_MAX\x10\x02\x1a\x02\x10\x01\x42\x13\n\x0f\x63om.tecton.dataP\x01'
)

_FEATURESTOREFORMATVERSION = _descriptor.EnumDescriptor(
  name='FeatureStoreFormatVersion',
  full_name='tecton_proto.data.FeatureStoreFormatVersion',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='FEATURE_STORE_FORMAT_VERSION_DEFAULT', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FEATURE_STORE_FORMAT_VERSION_TIME_NANOSECONDS', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FEATURE_STORE_FORMAT_VERSION_TTL_FIELD', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FEATURE_STORE_FORMAT_VERSION_MAX', index=3, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=b'\020\001',
  serialized_start=61,
  serialized_end=267,
)
_sym_db.RegisterEnumDescriptor(_FEATURESTOREFORMATVERSION)

FeatureStoreFormatVersion = enum_type_wrapper.EnumTypeWrapper(_FEATURESTOREFORMATVERSION)
FEATURE_STORE_FORMAT_VERSION_DEFAULT = 0
FEATURE_STORE_FORMAT_VERSION_TIME_NANOSECONDS = 1
FEATURE_STORE_FORMAT_VERSION_TTL_FIELD = 2
FEATURE_STORE_FORMAT_VERSION_MAX = 2


DESCRIPTOR.enum_types_by_name['FeatureStoreFormatVersion'] = _FEATURESTOREFORMATVERSION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)


DESCRIPTOR._options = None
_FEATURESTOREFORMATVERSION._options = None
# @@protoc_insertion_point(module_scope)



from pyspark.sql import DataFrame

from tecton_spark import errors
from tecton_spark.schema import Schema
from tecton_spark.schema_spark_utils import schema_from_spark


def validate_df_columns_and_feature_types(df: DataFrame, view_schema: Schema):
    df_columns = schema_from_spark(df.schema).column_name_raw_spark_types()
    df_column_names = frozenset([x[0] for x in df_columns])
    fv_columns = view_schema.column_name_raw_spark_types()
    fv_column_names = frozenset([x[0] for x in fv_columns])

    if fv_column_names.difference(df_column_names):
        raise errors.INGEST_DF_MISSING_COLUMNS(list(fv_column_names.difference(df_column_names)))
    for fv_column in fv_columns:
        if fv_column not in df_columns:
            raise errors.INGEST_COLUMN_TYPE_MISMATCH(
                fv_column[0], fv_column[1], [x for x in df_columns if x[0] == fv_column[0]][0][1]
            )
